@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21363 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-10-16 05:28:45 +0000 (Thu, 16 Oct 2008) $
  */
  
 #ifndef _SAFE_COLLECTION_H
@@ -240,9 +240,11 @@ class PSafeObject : public PObject
   private:
     mutable PMutex    safetyMutex;
     unsigned          safeReferenceCount;
-    PBoolean              safelyBeingRemoved;
+    bool              safelyBeingRemoved;
     PReadWriteMutex   safeInUseMutex;
     PReadWriteMutex * safeInUse;
+
+  friend class PSafeCollection;
 };
 
 
