@@ -26,14 +26,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20792 $
- * $Author: rjongbloed $
- * $Date: 2008-08-31 08:25:47 +0200 (dim 31 aoû 2008) $
+ * $Revision: 21373 $
+ * $Author: ms30002000 $
+ * $Date: 2008-10-16 19:00:01 +0000 (Thu, 16 Oct 2008) $
  */
 
 #include <ptlib.h>
 #include <vector>
 #include <map>
+#include <fstream>
 
 #include <ctype.h>
 #include <ptlib/pfactory.h>
@@ -41,13 +42,33 @@
 
 #ifdef _WIN32
 #include <ptlib/msos/ptlib/debstrm.h>
+#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
+namespace PWLibStupidLinkerHacks {
+  extern int loadVideoForWindowsStuff;
+};
+#endif
 #endif
 
-#ifdef __MACOSX__
-namespace PWLibStupidOSXHacks {
+#if defined(P_MINGW)
+namespace PWLibStupidLinkerHacks {
+  extern int loadFakeVideoStuff;
+#if defined(P_AUDIO)
+  extern int loadWindowsMultimediaStuff;
+#endif
+#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
+  extern int loadVideoForWindowsStuff;
+#endif
+#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
+  extern int loadDirectShowStuff;
+#endif
+};
+#endif
+     
+#ifdef P_MACOSX
+namespace PWLibStupidLinkerHacks {
+  extern int loadFakeVideoStuff;
   extern int loadShmVideoStuff;
   extern int loadCoreAudioStuff;
-  extern int loadFakeVideoStuff;
 };
 #endif
 
@@ -109,7 +130,7 @@ public:
   ostream       * stream;
   PTimeInterval   startTick;
   const char    * rolloverPattern;
-  unsigned        lastDayOfYear;
+  unsigned        lastRotate;
   ios::fmtflags   oldStreamFlags;
   std::streamsize oldPrecision;
 
@@ -158,8 +179,8 @@ PTHREAD_MUTEX_RECURSIVE_NP
     , stream(&cerr)
 #endif
     , startTick(PTimer::Tick())
-    , rolloverPattern("yyyy_MM_dd")
-    , lastDayOfYear(0)
+    , rolloverPattern("yyyy_MM_dd_hh_hh")
+    , lastRotate(0)
     , oldStreamFlags(ios::left)
     , oldPrecision(0)
   {
@@ -240,21 +261,19 @@ PTHREAD_MUTEX_RECURSIVE_NP
       PFilePath fn(filename);
       fn.Replace("%P", PString((unsigned int) PProcess::Current().GetProcessID()));
      
-      if ((options & PTrace::RotateDaily) != 0)
+      if ((options & PTrace::RotateLogMask) != 0)
       {
           PTime now;
           fn = PFilePath(fn.GetDirectory() + fn.GetTitle() + now.AsString(rolloverPattern, ((options&PTrace::GMTTime) ? PTime::GMT : PTime::Local)) + fn.GetType());
       }
 
-      PTextFile * traceOutput;
-      if (options & PTrace::AppendToFile) {
-        traceOutput = new PTextFile(fn, PFile::ReadWrite);
-        traceOutput->SetPosition(0, PFile::End);
-      }
+      ofstream * traceOutput;
+      if (options & PTrace::AppendToFile) 
+        traceOutput = new ofstream((const char *)fn, ios_base::out | ios_base::app);
       else 
-        traceOutput = new PTextFile(fn, PFile::WriteOnly);
+        traceOutput = new ofstream((const char *)fn, ios_base::out | ios_base::trunc);
 
-      if (traceOutput->IsOpen())
+      if (traceOutput->is_open())
         SetStream(traceOutput);
       else {
         PStringStream msgstrm;
@@ -286,17 +305,28 @@ void PTrace::Initialise(
   Initialise(level, filename, NULL, options);
 }
 
+static unsigned GetRotateVal(unsigned options)
+{
+  PTime now;
+  if (options & PTrace::RotateDaily)
+    return now.GetDayOfYear();
+  if (options & PTrace::RotateHourly) 
+    return now.GetHour();
+  if (options & PTrace::RotateMinutely)
+    return now.GetMinute();
+  return 0;
+}
+
 void PTrace::Initialise(unsigned level, const char * filename, const char * rolloverPattern, unsigned options)
 {
   PTraceInfo & info = PTraceInfo::Instance();
 
   info.options = options;
   info.thresholdLevel = level;
-  info.rolloverPattern = rolloverPattern != NULL ? rolloverPattern : "yyyy_MM_dd";
+  info.rolloverPattern = rolloverPattern != NULL ? rolloverPattern : "yyyy_MM_dd_hh_mm";
   // Does PTime::GetDayOfYear() etc. want to take zone param like PTime::AsString() to switch 
   // between os_gmtime and os_localtime?
-  info.lastDayOfYear = (options & RotateDaily) != 0 ? PTime().GetDayOfYear() : 0;
-
+  info.lastRotate = GetRotateVal(options);
   info.OpenTraceFile(filename);
 
 #if PTRACING
@@ -357,11 +387,11 @@ ostream & PTrace::Begin(unsigned level, const char * fileName, int lineNum)
 
   info.Lock();
 
-  if ((info.filename != NULL) && (info.options&RotateDaily) != 0) {
-    unsigned day = PTime().GetDayOfYear();
-    if (day != info.lastDayOfYear) {
+  if ((info.filename != NULL) && (info.options&RotateLogMask) != 0) {
+    unsigned rotateVal = GetRotateVal(info.options);
+    if (rotateVal != info.lastRotate) {
       info.OpenTraceFile(NULL);
-      info.lastDayOfYear = day;
+      info.lastRotate = rotateVal;
       if (info.stream == NULL)
         info.SetStream(&cerr);
     }
@@ -808,7 +838,9 @@ void PTimerList::QueueRequest(RequestType::Action action, PTimer * timer, bool _
           // if the timer is a new timer, then use special queue as asynchronous add won't work
           if (r == activeTimers.end()) {
             RequestType request(action, timer);
-            addQueue.push_back(request);
+            queueMutex.Wait();
+            addQueue.push(request);
+            queueMutex.Signal();
           }
         }
         break;
@@ -833,10 +865,9 @@ void PTimerList::QueueRequest(RequestType::Action action, PTimer * timer, bool _
     isSync = true;
   }
 
-  {
-    PWaitAndSignal m(queueMutex);
-    requestQueue.push(request);
-  }
+  queueMutex.Wait();
+  requestQueue.push(request);
+  queueMutex.Signal();
 
   PProcess::Current().SignalTimerChange();
 
@@ -851,28 +882,28 @@ PTimeInterval PTimerList::Process()
   PWaitAndSignal l(timerListMutex);
 
   // process the requests in the timer request queue
-  {
-    PWaitAndSignal q(queueMutex);
-    while (requestQueue.size() > 0) {
-      RequestType request = requestQueue.front();
-      requestQueue.pop();
-      TimerInfoMapType::iterator r = activeTimers.find(request.id);
-      switch (request.action) {
-        case PTimerList::RequestType::Start:
-          if (r == activeTimers.end()) 
-            activeTimers.insert(TimerInfoMapType::value_type(request.id, TimerInfoType(request.timer)));
-          break;
-        case PTimerList::RequestType::Stop:
-          if (r != activeTimers.end()) 
-            activeTimers.erase(r);
-          break;
-        default:
-          PAssertAlways("unknown timer request code");
-          break;
-      }
-      if (request.sync != NULL)
-        request.sync->Signal();
+  while (!requestQueue.empty()) {
+    queueMutex.Wait();
+    RequestType request = requestQueue.front();
+    requestQueue.pop();
+    queueMutex.Signal();
+
+    TimerInfoMapType::iterator r = activeTimers.find(request.id);
+    switch (request.action) {
+      case PTimerList::RequestType::Start:
+        if (r == activeTimers.end()) 
+          activeTimers.insert(TimerInfoMapType::value_type(request.id, TimerInfoType(request.timer)));
+        break;
+      case PTimerList::RequestType::Stop:
+        if (r != activeTimers.end()) 
+          activeTimers.erase(r);
+        break;
+      default:
+        PAssertAlways("unknown timer request code");
+        break;
     }
+    if (request.sync != NULL)
+      request.sync->Signal();
   }
 
   // calculate interval since last processing, and update time of last processing to now
@@ -916,11 +947,14 @@ PTimeInterval PTimerList::Process()
   }
 
   // add in an new timers created while timers were processed
-  while (addQueue.size() > 0) {
-    RequestType & request = addQueue[0];
+  while (!addQueue.empty()) {
+    queueMutex.Wait();
+    RequestType request = addQueue.front();
+    addQueue.pop();
+    queueMutex.Signal();
+
     activeTimers.insert(TimerInfoMapType::value_type(request.id, TimerInfoType(request.timer)));
-    addQueue.erase(addQueue.begin());
-    request.timer->Process(sampleTime, minTimeLeft);
+    request.timer->Process(0, minTimeLeft);
   }
 
   return minTimeLeft;
@@ -1489,6 +1523,11 @@ PProcess::PProcess(const char * manuf, const char * name,
         executableFile = execFile;
     }
   }
+
+#if defined(P_VIDEO)  && defined(P_VFW_CAPTURE)
+  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
+#endif
+
 #endif // _WIN32
 
   if (productName.IsEmpty())
@@ -1500,20 +1539,40 @@ PProcess::PProcess(const char * manuf, const char * name,
 
   Construct();
 
-#ifdef __MACOSX__
-  
+#if defined(P_MINGW)
 #ifdef P_VIDEO
-  PWLibStupidOSXHacks::loadFakeVideoStuff = 1;
+  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
+#endif
+#if defined(P_AUDIO)
+  PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
+#endif
+#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
+  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
+#endif
+#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
+  PWLibStupidLinkerHacks::loadDirectShowStuff = 1;
+#endif
+
+#endif
+
+
+#ifdef P_MACOSX
+
+#ifdef P_VIDEO
+
+  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
+  
 #ifdef P_SHM_VIDEO
-  PWLibStupidOSXHacks::loadShmVideoStuff = 1;
+  PWLibStupidLinkerHacks::loadShmVideoStuff = 1;
 #endif // P_SHM_VIDEO
-#endif // P_VIDEO
+
+#endif
   
 #ifdef P_AUDIO
-  PWLibStupidOSXHacks::loadCoreAudioStuff = 1;
+  PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
 #endif // P_AUDIO
   
-#endif // __MACOSX__
+#endif // P_MACOSX
 
   // create one instance of each class registered in the 
   // PProcessStartup abstract factory
