@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20846 $
+ * $Revision: 21322 $
  * $Author: rjongbloed $
- * $Date: 2008-09-04 12:23:18 +0000 (Thu, 04 Sep 2008) $
+ * $Date: 2008-10-14 06:43:48 +0000 (Tue, 14 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -488,14 +488,23 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
 {
   mutex.Wait();
 
-  PCaselessString key = name;
+  PString key = name;
+  PINDEX len = key.GetLength();
 
   // Check for a legal hostname as per RFC952
   if (key.IsEmpty() ||
-      key.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.") != P_MAX_INDEX ||
-      key[key.GetLength()-1] == '-' ||
-      !isalpha(key[0]))
+      key.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.") != P_MAX_INDEX ||
+      key[len-1] == '-' ||
+      !isalpha(key[0])) {
+    PTRACE(3, "Socket\tIllegal RFC952 characters in DNS name \"" << key << '"');
     return NULL;
+  }
+
+  // We lowercase this way rather than toupper() as that is locale dependent, and DNS names aren't.
+  for (PINDEX i = 0; i < len; i++) {
+    if (key[i] >= 'a')
+      key[i] &= 0x5f;
+  }
 
   PIPCacheData * host = GetAt(key);
   int localErrNo = NO_DATA;
@@ -588,7 +597,7 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
   }
 
   if (host->GetHostAddress() == 0) {
-    PTRACE(5, "Socket\tName lookup of \"" << name << "\" failed: errno=" << localErrNo);
+    PTRACE(4, "Socket\tName lookup of \"" << name << "\" failed: errno=" << localErrNo);
     return NULL;
   }
 
@@ -1472,7 +1481,7 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 #endif
   }
   
-#ifndef __BEOS__
+#ifndef P_BEOS
   // attempt to listen
   if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
     os_close();
@@ -1799,49 +1808,52 @@ PString PIPSocket::Address::AsString() const
 }
 
 
-PBoolean PIPSocket::Address::FromString(const PString & dotNotation)
+PBoolean PIPSocket::Address::FromString(const PString & ipAndInterface)
 {
   version = 0;
   memset(&v, 0, sizeof(v));
 
+  PINDEX percent = ipAndInterface.Find('%');
+  PString dotNotation = ipAndInterface.Left(percent);
+  if (!dotNotation.IsEmpty()) {
 #if P_HAS_IPV6
 
-  struct addrinfo *res = NULL;
-  struct addrinfo hints = { AI_NUMERICHOST, PF_UNSPEC }; // Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
+    struct addrinfo *res = NULL;
+    struct addrinfo hints = { AI_NUMERICHOST, PF_UNSPEC }; // Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
 
-  if (getaddrinfo((const char *)dotNotation, NULL , &hints, &res) == 0) {
-    if (res->ai_family == PF_INET6) {
-      // IPv6 addr
-      version = 6;
-      struct sockaddr_in6 * addr_in6 = (struct sockaddr_in6 *)res->ai_addr;
-      v.six = addr_in6->sin6_addr;
-    } else {
-      // IPv4 addr
-      version = 4;
-      struct sockaddr_in * addr_in = (struct sockaddr_in *)res->ai_addr;
-      v.four = addr_in->sin_addr;
+    if (getaddrinfo((const char *)dotNotation, NULL , &hints, &res) == 0) {
+      if (res->ai_family == PF_INET6) {
+        // IPv6 addr
+        version = 6;
+        struct sockaddr_in6 * addr_in6 = (struct sockaddr_in6 *)res->ai_addr;
+        v.six = addr_in6->sin6_addr;
+      } else {
+        // IPv4 addr
+        version = 4;
+        struct sockaddr_in * addr_in = (struct sockaddr_in *)res->ai_addr;
+        v.four = addr_in->sin_addr;
+      }
+      freeaddrinfo(res);
+      return IsValid();
     }
-    freeaddrinfo(res);
-    return IsValid();
-  }
 
 #else //P_HAS_IPV6
 
-  DWORD iaddr;
-  if (dotNotation.FindSpan("0123456789.") == P_MAX_INDEX &&
-                    (iaddr = ::inet_addr((const char *)dotNotation)) != (DWORD)INADDR_NONE) {
-    version = 4;
-    v.four.s_addr = iaddr;
-    return true;
-  }
+    DWORD iaddr;
+    if (dotNotation.FindSpan("0123456789.") == P_MAX_INDEX &&
+                      (iaddr = ::inet_addr((const char *)dotNotation)) != (DWORD)INADDR_NONE) {
+      version = 4;
+      v.four.s_addr = iaddr;
+      return true;
+    }
 
 #endif
+  }
 
-  PINDEX percent = dotNotation.Find('%');
   if (percent == P_MAX_INDEX)
     return false;
 
-  PString iface = dotNotation.Mid(percent+1);
+  PString iface = ipAndInterface.Mid(percent+1);
   if (iface.IsEmpty())
     return false;
 
@@ -2330,7 +2342,7 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
 
   PBoolean broadcast = addr.IsAny() || addr.IsBroadcast();
   if (broadcast) {
-#ifdef __BEOS__
+#ifdef P_BEOS
     PAssertAlways("Broadcast option under BeOS is not implemented yet");
     return PFalse;
 #else
@@ -2399,7 +2411,7 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
   
 #endif // P_MACOSX
 
-#ifndef __BEOS__
+#ifndef P_BEOS
   if (broadcast)
     SetOption(SO_BROADCAST, 0);
 #endif
