@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20613 $
- * $Author: rjongbloed $
- * $Date: 2008-07-24 09:26:23 +0200 (jeu 24 jui 2008) $
+ * $Revision: 21176 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-27 20:34:32 +0000 (Sat, 27 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -48,6 +48,12 @@
 #include <ptlib/wm/mmsystemx.h>
 #endif
 
+#if defined(P_MINGW)
+namespace PWLibStupidLinkerHacks {
+   int loadWindowsMultimediaStuff;
+};
+#endif
+     
 class PSound;
 
 PCREATE_SOUND_PLUGIN(WindowsMultimedia, PSoundChannelWin32);
@@ -686,25 +692,6 @@ PStringArray PSoundChannelWin32::GetDeviceNames(Directions dir)
 }
 
 
-PString PSoundChannelWin32::GetDefaultDevice(Directions dir)
-{
-  RegistryKey registry("HKEY_CURRENT_USER\\Software\\Microsoft\\Multimedia\\Sound Mapper",
-                       RegistryKey::ReadOnly);
-
-  PString str;
-
-  if (dir == Player) {
-    if (!registry.QueryValue("Playback", str))
-      return GetWaveOutDeviceName(WAVE_MAPPER);
-  }
-  else {
-    if (!registry.QueryValue("Record", str))
-      return GetWaveInDeviceName(WAVE_MAPPER);
-  }
-
-  return str.Trim();
-}
-
 PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir, unsigned& id)
 {
   PINDEX offset = device.Find(PDevicePluginServiceDescriptor::SeparatorChar);
