@@ -12,9 +12,9 @@
  * compliance with the License. You may obtain a copy of the License at
  * http://www.mozilla.org/MPL/
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Revision: 21176 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-27 20:34:32 +0000 (Sat, 27 Sep 2008) $
  */
 
 #define P_FORCE_STATIC_PLUGIN
@@ -24,8 +24,8 @@
 #include <ptlib/vconvert.h>
 #include <ptlib/unix/ptlib/shmvideo.h>
 
-#ifdef __MACOSX__
-namespace PWLibStupidOSXHacks {
+#ifdef P_MACOSX
+namespace PWLibStupidLinkerHacks {
 	int loadShmVideoStuff;
 };
 
