@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Revision: 21137 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-22 16:57:19 +0000 (Mon, 22 Sep 2008) $
  */
 
 #pragma implementation "sockets.h"
@@ -80,7 +80,7 @@
 #include <bsp.h>
 #endif
 
-#ifdef __BEOS__
+#ifdef P_BEOS
 #include <posix/sys/ioctl.h> // for FIONBIO
 #include <be/bone/net/if.h> // for ifconf
 #include <be/bone/sys/sockio.h> // for SIOCGI*
@@ -162,8 +162,17 @@ static int SetNonBlocking(int fd)
 
 int PSocket::os_socket(int af, int type, int protocol)
 {
+  int fd = ::socket(af, type, protocol);
+
+#if P_HAS_RECVMSG
+  if ((fd != -1) && (type == SOCK_DGRAM)) {
+    int v = 1;
+    setsockopt(fd, IPPROTO_IP, IP_RECVERR, &v, sizeof(v));
+  }
+#endif
+
   // attempt to create a socket
-  return SetNonBlocking(PX_NewHandle(GetClass(), ::socket(af, type, protocol)));
+  return SetNonBlocking(PX_NewHandle(GetClass(), fd));
 }
 
 
@@ -214,7 +223,7 @@ PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, PINDEX *
 }
 
 
-#if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS) && !defined(__BEOS__)
+#if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS) && !defined(P_BEOS)
 
 PChannel::Errors PSocket::Select(SelectList & read,
                                  SelectList & write,
@@ -547,16 +556,6 @@ PBoolean PSocket::os_recvfrom(
   if (!PXSetIOBlock(PXReadBlock, readTimeout))
     return PFalse;
 
-  // if we don't care what interface the packet arrives on, then don't bother getting the information
-  if (!catchReceiveToAddr) {
-    int r = ::recvfrom(os_handle, (char *)buf, len, flags, (sockaddr *)addr, (socklen_t *)addrlen);
-    if (!ConvertOSError(r, LastReadError))
-      return PFalse;
-
-    lastReadCount = r;
-    return lastReadCount > 0;
-  }
-
   msghdr readData;
   memset(&readData, 0, sizeof(readData));
 
@@ -574,7 +573,12 @@ PBoolean PSocket::os_recvfrom(
   readData.msg_controllen = sizeof(auxdata);
 
   // read a packet 
-  int r = ::recvmsg(os_handle, &readData, 0);
+  int r = ::recvmsg(os_handle, &readData, flags);
+  if (r == -1) {
+    PTRACE(5, "PWLIB\trecvmsg returned error " << r);
+    ::recvmsg(os_handle, &readData, MSG_ERRQUEUE);
+  }
+
   if (!ConvertOSError(r, LastReadError))
     return PFalse;
 
@@ -586,7 +590,6 @@ PBoolean PSocket::os_recvfrom(
       if (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_PKTINFO) {
         in_pktinfo * info = (in_pktinfo *)CMSG_DATA(cmsg);
         SetLastReceiveAddr(&info->ipi_spec_dst, sizeof(in_addr));
-        break;
       }
     }
   }
@@ -667,7 +670,9 @@ PBoolean PSocket::Read(void * buf, PINDEX len)
   if (!PXSetIOBlock(PXReadBlock, readTimeout)) 
     return PFalse;
 
-  if (ConvertOSError(lastReadCount = ::recv(os_handle, (char *)buf, len, 0)))
+  int lastReadCount = ::recv(os_handle, (char *)buf, len, 0);
+    return lastReadCount > 0;
+  if (ConvertOSError(lastReadCount))
     return lastReadCount > 0;
 
   lastReadCount = 0;
@@ -1597,11 +1602,11 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
             memcpy(&ifReq, ifName, sizeof(ifreq));
             if (ioctl(sock.GetHandle(), SIOCGIFNETMASK, &ifReq) >= 0) {
               PIPSocket::Address mask = 
-#ifndef __BEOS__
+#ifndef P_BEOS
     ((sockaddr_in *)&ifReq.ifr_netmask)->sin_addr;
 #else
     ((sockaddr_in *)&ifReq.ifr_mask)->sin_addr;
-#endif // !__BEOS__
+#endif // !P_BEOS
               PINDEX i;
               for (i = 0; i < list.GetSize(); i++) {
 #ifdef P_TORNADO
